#!/usr/bin/env python3

class command():
    def __init__(self):
        pass
    
    def initialize(self, environment):
        self.env = environment
    
    def shutdown(self):
        pass
    
    def getDescription(self):
        return "Apply last previewed voice to current session (temporary)"
    
    def run(self):
        # Check if we have a tested voice stored (must be tested first)
        if ('commandBuffer' not in self.env or 
            'lastTestedModule' not in self.env['commandBuffer'] or
            'lastTestedVoice' not in self.env['commandBuffer']):
            self.env['runtime']['outputManager'].presentText("No voice has been tested yet", interrupt=True)
            self.env['runtime']['outputManager'].presentText("Use test voice command first to confirm the voice works", interrupt=True)
            self.env['runtime']['outputManager'].playSound('Cancel')
            return
        
        module = self.env['commandBuffer']['lastTestedModule']
        voice = self.env['commandBuffer']['lastTestedVoice']
        
        self.env['runtime']['outputManager'].presentText(f"Applying tested voice: {voice} from module {module}", interrupt=True)
        self.env['runtime']['outputManager'].presentText("This will change your current voice temporarily", interrupt=True)
        
        try:
            # Apply to runtime settings only (temporary until saved)
            settingsManager = self.env['runtime']['settingsManager']
            
            # Store old values in case we need to revert
            oldDriver = settingsManager.getSetting('speech', 'driver')
            oldModule = settingsManager.getSetting('speech', 'module')
            oldVoice = settingsManager.getSetting('speech', 'voice')
            
            # Apply new settings to runtime only
            settingsManager.settings['speech']['driver'] = 'speechdDriver'
            settingsManager.settings['speech']['module'] = module
            settingsManager.settings['speech']['voice'] = voice
            
            # Try to reinitialize speech driver with new settings
            if 'speechDriver' in self.env['runtime']:
                try:
                    speechDriver = self.env['runtime']['speechDriver']
                    speechDriver.shutdown()
                    speechDriver.initialize(self.env)
                    
                    # Test the new voice
                    self.env['runtime']['outputManager'].presentText("Voice applied successfully. This is how it sounds.", interrupt=False)
                    self.env['runtime']['outputManager'].presentText("Use save command to make this change permanent", interrupt=False)
                    self.env['runtime']['outputManager'].playSound('Accept')
                    
                except Exception as e:
                    # Revert on failure
                    settingsManager.settings['speech']['driver'] = oldDriver
                    settingsManager.settings['speech']['module'] = oldModule
                    settingsManager.settings['speech']['voice'] = oldVoice
                    
                    self.env['runtime']['outputManager'].presentText(f"Failed to apply voice: {str(e)}", interrupt=False)
                    self.env['runtime']['outputManager'].presentText("Reverted to previous settings", interrupt=False)
                    self.env['runtime']['outputManager'].playSound('Error')
            else:
                self.env['runtime']['outputManager'].presentText("Speech driver not available", interrupt=False)
                self.env['runtime']['outputManager'].playSound('Error')
                
        except Exception as e:
            self.env['runtime']['outputManager'].presentText(f"Error applying voice: {str(e)}", interrupt=False)
            self.env['runtime']['outputManager'].playSound('Error')
    
    def setCallback(self, callback):
        pass